/***********************************************************
 * $Id$
 *
 * FancyDB
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDBCONNECTION_H_
#define FANCYDBCONNECTION_H_

#include <fancydb.h>
#include <fancydbdriver.h>

#include <fancypoolableconnection.h>

namespace Fancy {

namespace DB {

class Driver;
class Statement;
class PreparedStatement;
class Transaction;

class FANCYDB_API Connection: public virtual Fancy::PoolableConnection {
public:
	typedef Fancy::ObjPtr<Connection> ptr_t;


	/**
	 * See
	 *   http://en.wikipedia.org/wiki/Isolation_(database_systems)#Isolation_levels
	 *   http://dev.mysql.com/doc/refman/5.1/en/set-transaction.html
	 *   http://technet.microsoft.com/de-de/library/ms173763.aspx
	 *   http://www.postgresql.org/docs/9.2/static/sql-set-transaction.html
	 *   http://db.apache.org/derby/docs/10.4/devguide/cdevconcepts15366.html
	 */
	enum TransactionIsolationLevel {
		DefaultIsolationLevel = 0, // do not set any specific transaction isolation level
		RepeatableRead = 1,
		ReadCommited = 2,
		ReadUncommited = 3,
		Serializable = 4,
		SnapShot = 5 // MSSQL 2005+
	};


	/**
	 * Start a transaction in this connection, without thinking about TransactionIsolationLevel.
	 * This method will throw an exception, if there is another transaction active on this connection.
	 * Usual implementations will call startTransaction(DefaultIsolationLevel).
	 */
	virtual Fancy::ObjPtr<Transaction> startTransaction() = 0;

	/**
	 * Start a transaction in this connection, using the specified TransactionIsolationLevel.
	 *
	 * Note: Not all database systems resp. FancyDB drivers will support all isolaton levels.
	 * if there is another transaction active on this connection.
	 *
	 * This method will throw an exception,if there is another transaction active on this connection,
	 * or if the isolation level is known to be unsupported.
	 */
	virtual Fancy::ObjPtr<Transaction> startTransaction(TransactionIsolationLevel isolationLevel) = 0;

	/**
	 * Create a simple SQL statement to execute.
	 */
	virtual Fancy::ObjPtr<Statement> createStatement() = 0;

	/**
	 * Create a simple SQL statement to execute and set the sql string.
	 */
	virtual Fancy::ObjPtr<Statement> createStatement(const std::string &sql) = 0;

	/**
	 * Create a Prepared Statement to execute.
	 */
	virtual Fancy::ObjPtr<PreparedStatement> createPreparedStatement() = 0;

	/**
	 * Create a Prepared Statement to execute and set the sql string.
	 */
	virtual Fancy::ObjPtr<PreparedStatement> createPreparedStatement(const std::string &sql) = 0;

	/**
	 * @return The underlying driver instance.
	 */
	virtual Fancy::ObjPtr<Driver> getDriver() const = 0;

	/**
	 * Quote a string for insertion into a simple SQL statement, including the ' before and after.
	 *
	 * @param s A string to append the quoted string to.
	 * @param data The pointer to the content to be quoted.
	 * @param data_len The length of the dat to be quoted.
	 */
	virtual void quoteString(std::string &s, const char *data, size_t data_len) const = 0;

  /**
   * Quote a byte array for insertion into a simple SQL statement, including the ' before and after.
   *
   * @param s A string to append the quoted string to.
   * @param data The pointer to the content to be quoted.
   * @param data_len The length of the dat to be quoted.
   */
  virtual void quoteBytes(std::string &s, const char *data, size_t data_len) const = 0;

  /**
	 * @return The default format for @c UtcTimestamp.format() used to bind
	 *         placeholders.
	 */
	virtual const char *getDefaultTimestampFormat() const = 0;

protected:
	Connection();
	virtual ~Connection();
};

}

}

#endif /* CONNECTION_H_ */
