/***********************************************************
 * $Id$
 *
 * FancyDB
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYDBPREPAREDSTATEMENT_H_
#define FANCYDBPREPAREDSTATEMENT_H_

#include <fancydbstatement.h>

namespace Fancy {

class UtcTimestamp;
class String;
class Double;
class Integer;
class Long;
template<class T> class ArrayObject;

namespace DB {

/**
 * A prepared statement, which allow setting the values of placeholders.
 *
 * There is no @c setNull method, because most database-specific implementations
 * depend on the fact, that the same placeholder is bound to the same value type
 * for each call to @c execute().
 *
 * Most implementations force, that you have called any of the type-specific @c set*
 * methods before the first call to @c execute in order to determine the type
 * of the placeholder. Such implementation likely bind the placeholders during the
 * first call to @c execute(), because they assume, that the can not determine the
 * client-side placeholder type before this point in time.
 */
class FANCYDB_API PreparedStatement: public virtual Statement {
public:
	typedef Fancy::ObjPtr<PreparedStatement> ptr_t;

	/**
	 * A new instance, which mimics a prepared statement, which delegates
	 * to the given statement. A generic prepared statement may not be suitable
	 * for all database engines, because string and BLOB values are quoted using
	 * @c Connection::quoteString resp. @c Connection::quoteBytes.
	 */
	static ptr_t newGenericInstance(const Statement::ptr_t &statement);

	/**
	 * @return The number of placeholders in the statement set by
	 *         @c setSQL().
	 */
	virtual size_t getPlaceholderCount() const = 0;

	/**
	  * placeholder = 0..n
	  */
	virtual void setInt(size_t placeholder, int32_t i) = 0;

	/**
	  * placeholder = 0..n
	  */
	virtual void setInt(size_t placeholder, const Fancy::ObjPtr<Fancy::Integer> &i) = 0;

	/**
	  * placeholder = 0..n
	  */
	virtual void setLong(size_t placeholder, int64_t i) = 0;

	/**
	  * placeholder = 0..n
	  */
	virtual void setLong(size_t placeholder, const Fancy::ObjPtr<Fancy::Long> &i) = 0;

	/**
	  * placeholder = 0..n
	  */
	virtual void setDouble(size_t placeholder, double x) = 0;

	/**
	  * placeholder = 0..n
	  */
	virtual void setDouble(size_t placeholder, const Fancy::ObjPtr<Fancy::Double> &x) = 0;

	/**
	  * placeholder = 0..n
	  */
	virtual void setTimestamp(size_t placeholder, const Fancy::ObjPtr<Fancy::UtcTimestamp> &ts) = 0;

	/**
	  * placeholder = 0..n
	  */
	virtual void setString(size_t placeholder, const std::string &s) = 0;

	/**
	  * placeholder = 0..n
	  */
	virtual void setString(size_t placeholder, const Fancy::ObjPtr<Fancy::String> &s) = 0;

	/**
	  * placeholder = 0..n
	  */
	virtual void setBlob(size_t placeholder, const Fancy::ObjPtr<Fancy::ArrayObject<int8_t> > &b) = 0;

	/**
	 * Add the current set of values to the batch being performed using @excuteUpdate().
	 */
	virtual void addBatch() = 0;

	/**
	 * Get the number of batches value sets, aka the number of @c addBatch() calls since the
	 * last call to @c executeUpdate().
	 */
	virtual size_t getBatchSize() const = 0;

	/**
	 * Get the number of all batches values submitted via @c executeUpdate() since the
	 * last call to @c setSQL().
	 */
	virtual size_t getTotalBatchSize() const = 0;

protected:
	PreparedStatement();
	virtual ~PreparedStatement();
};

}

}

#endif /* FANCYDBPREPAREDSTATEMENT_H_ */
